@@ -17,28 +17,28 @@ SplineBasis<-function(knots, order=4, keep.duplicates=FALSE) {
 	}
 	new("SplineBasis", knots=knots, order=as.integer(order), Matrices=M2)
 }
-OrthogonalizeBasis<-function(object,...){
-	obase<-new("OrthogonalSplineBasis",object)
+GramMatrix<-function(object){
 	M<-object@Matrices
 	k<-object@order
 	Delta<-1/Hankel(1:(2*k-1),k,k)
-	
 	di<-diff(object@knots[(k):(length(object@knots)-k+1)])
 	d<-dim(M)
 	s<-matrix(0,d[2],d[2])
 	for(i in 1:d[3]) { 
 		s<-s+di[i]*t(M[,,i])%*%Delta%*%M[,,i]
 	}
+	return(s)
+}
+OrthogonalizeBasis<-function(object,...){
+	s<-GramMatrix(object)
 	L<-solve(chol(s))
+	M<-object@Matrices
 	N<-M
+	d<-dim(M)
 	for( i in 1:d[3]) { 
 		N[,,i]<-M[,,i]%*%L
 	}
-	
-	obase@Matrices<-N
-	obase@transformation<-L
-
-	return(obase)
+	return(new("OrthogonalSplineBasis", knots=object@knots, order=as.integer(object@order), Matrices=N, transformation=L))
 }
 OrthogonalSplineBasis<-function(knots,...)OrthogonalizeBasis(SplineBasis(knots,...))
 setGeneric("orthogonalize",function(object,...)standardGeneric("orthogonalize"))
@@ -55,7 +55,7 @@ EvaluateBasis<-function(object,x,...) {
 		knots<-object@knots
 		order<-object@order
 		if(x < knots[order] | x>knots[length(knots)-order+1])return(rep(NA,dim(object)[2]))
-		ind<- x<knots
+		if(x==knots[length(knots)-order+1]){ ind <- x<=knots } else { ind <- x<knots }
 		if(all(ind)|all(!ind))  {
 			if(x==knots[length(knots)-order+1]) 
 				return(rep(1,order)%*%matrix(M[,,dim(M)[3]],nrow=order)) 
